@@ -21,16 +21,14 @@
  * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
  *                                                                  *
 \********************************************************************/
-extern "C"
-{
+#include <glib.h>
+
 #include <config.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include "gncBillTermP.h"
 #include "gncInvoiceP.h"
-}
-#include <glib.h>
 
 #include "gnc-xml-helper.h"
 #include "sixtp.h"
@@ -384,11 +382,8 @@ static gboolean
 invoice_tochargeamt_handler (xmlNodePtr node, gpointer invoice_pdata)
 {
     struct invoice_pdata* pdata = static_cast<decltype (pdata)> (invoice_pdata);
-    gnc_numeric* num = dom_tree_to_gnc_numeric (node);
-    g_return_val_if_fail (num, FALSE);
 
-    gncInvoiceSetToChargeAmount (pdata->invoice, *num);
-    g_free (num);
+    gncInvoiceSetToChargeAmount (pdata->invoice, dom_tree_to_gnc_numeric (node));
     return TRUE;
 }
 
