@@ -171,7 +171,7 @@ string_to_gint64 (const gchar* str, gint64* v)
     g_return_val_if_fail (str, FALSE);
 
     /* must use "<" here because %n's effects aren't well defined */
-    if (sscanf (str, " " QOF_SCANF_LLD "%n", &v_in, &num_read) < 1)
+    if (sscanf (str, " %lld%n", &v_in, &num_read) < 1)
     {
         return (FALSE);
     }
