@@ -1101,7 +1101,8 @@ qof_strftime(gchar *buf, gsize max, const gchar *format, const struct tm *tm)
 gchar *
 gnc_date_timestamp (void)
 {
-    return g_strdup(GncDateTime::timestamp().c_str());
+    auto timestamp = GncDateTime::timestamp();
+    return g_strdup(timestamp.c_str());
 }
 
 /********************************************************************\
@@ -1328,19 +1329,19 @@ gnc_time64_get_day_end (time64 time_val)
 void
 gnc_tm_get_today_start (struct tm *tm)
 {
-    gnc_tm_get_day_start(tm, time(NULL));
+    gnc_tm_get_day_start(tm, gnc_time(nullptr));
 }
 
 void
 gnc_tm_get_today_neutral (struct tm *tm)
 {
-    gnc_tm_get_day_neutral(tm, time(NULL));
+    gnc_tm_get_day_neutral(tm, gnc_time(nullptr));
 }
 
 void
 gnc_tm_get_today_end (struct tm *tm)
 {
-    gnc_tm_get_day_end(tm, time(NULL));
+    gnc_tm_get_day_end(tm, gnc_time(nullptr));
 }
 
 time64
@@ -1348,7 +1349,7 @@ gnc_time64_get_today_start (void)
 {
     struct tm tm;
 
-    gnc_tm_get_day_start(&tm, time(NULL));
+    gnc_tm_get_day_start(&tm, gnc_time(nullptr));
     return gnc_mktime(&tm);
 }
 
@@ -1357,7 +1358,7 @@ gnc_time64_get_today_end (void)
 {
     struct tm tm;
 
-    gnc_tm_get_day_end(&tm, time(NULL));
+    gnc_tm_get_day_end(&tm, gnc_time(nullptr));
     return gnc_mktime(&tm);
 }
 
@@ -1378,36 +1379,22 @@ gnc_dow_abbrev(gchar *buf, int buf_len, int dow)
  *  GValue handling
  ********************************************************************/
 
-static gpointer
-time64_boxed_copy_func (gpointer in_time64)
+static Time64*
+time64_boxed_copy_func (Time64 *in_time64)
 {
-    Time64* newvalue;
-
-    newvalue = static_cast<Time64*>(g_malloc (sizeof (Time64)));
-    memcpy (newvalue, in_time64, sizeof(Time64));
+    Time64* newvalue = g_new (Time64, 1);
+    *newvalue = *in_time64;
 
     return newvalue;
 }
 
 static void
-time64_boxed_free_func (gpointer in_time64)
+time64_boxed_free_func (Time64 *in_time64)
 {
     g_free (in_time64);
 }
 
-GType
-time64_get_type( void )
-{
-    static GType type = 0;
-
-    if ( type == 0 )
-    {
-        type = g_boxed_type_register_static( "time64",
-                                             time64_boxed_copy_func,
-                                             time64_boxed_free_func );
-    }
-    return type;
-}
+G_DEFINE_BOXED_TYPE (Time64, time64, time64_boxed_copy_func, time64_boxed_free_func)
 
 /* ================================================= */
 
