@@ -33,7 +33,6 @@
 
 #include <glib.h>
 #include <glib/gi18n.h>
-#include <string.h>		/* for memcpy() */
 #include <qofinstance-p.h>
 
 #include "gncCustomerP.h"
@@ -204,6 +203,29 @@ GncOwnerType gncOwnerGetType (const GncOwner *owner)
     return owner->type;
 }
 
+const char * gncOwnerGetTypeString (const GncOwner *owner)
+{
+    GncOwnerType type = gncOwnerGetType(owner);
+    switch (type)
+    {
+    case GNC_OWNER_NONE:
+        return N_("None");
+    case GNC_OWNER_UNDEFINED:
+        return N_("Undefined");
+    case GNC_OWNER_CUSTOMER:
+        return N_("Customer");
+    case GNC_OWNER_JOB:
+        return N_("Job");
+    case GNC_OWNER_VENDOR:
+        return N_("Vendor");
+    case GNC_OWNER_EMPLOYEE:
+        return N_("Employee");
+    default:
+        PWARN ("Unknown owner type");
+        return NULL;
+    }
+}
+
 QofIdTypeConst
 qofOwnerGetType(const GncOwner *owner)
 {
@@ -376,7 +398,7 @@ void gncOwnerCopy (const GncOwner *src, GncOwner *dest)
 {
     if (!src || !dest) return;
     if (src == dest) return;
-    memcpy (dest, src, sizeof (*dest));
+    *dest = *src;
 }
 
 gboolean gncOwnerEqual (const GncOwner *a, const GncOwner *b)
@@ -641,9 +663,11 @@ gboolean gncOwnerGetOwnerFromLot (GNCLot *lot, GncOwner *owner)
         gncOwnerInitJob (owner, gncJobLookup (book, guid));
         break;
     default:
+        guid_free (guid);
         return FALSE;
     }
 
+    guid_free (guid);
     return (owner->owner.undefined != NULL);
 }
 
@@ -704,7 +728,7 @@ gint
 gncOwnerLotsSortFunc (GNCLot *lotA, GNCLot *lotB)
 {
     GncInvoice *ia, *ib;
-    Timespec da, db;
+    time64 da, db;
 
     ia = gncInvoiceGetInvoiceFromLot (lotA);
     ib = gncInvoiceGetInvoiceFromLot (lotB);
@@ -712,29 +736,31 @@ gncOwnerLotsSortFunc (GNCLot *lotA, GNCLot *lotB)
     if (ia)
         da = gncInvoiceGetDateDue (ia);
     else
-        da = xaccTransRetDatePostedTS (xaccSplitGetParent (gnc_lot_get_earliest_split (lotA)));
+        da = xaccTransRetDatePosted (xaccSplitGetParent (gnc_lot_get_earliest_split (lotA)));
 
     if (ib)
         db = gncInvoiceGetDateDue (ib);
     else
-        db = xaccTransRetDatePostedTS (xaccSplitGetParent (gnc_lot_get_earliest_split (lotB)));
+        db = xaccTransRetDatePosted (xaccSplitGetParent (gnc_lot_get_earliest_split (lotB)));
 
-    return timespec_cmp (&da, &db);
+    return (da > db) - (da < db);
 }
 
 GNCLot *
-gncOwnerCreatePaymentLot (const GncOwner *owner, Transaction **preset_txn,
-                          Account *posted_acc, Account *xfer_acc,
-                          gnc_numeric amount, gnc_numeric exch, Timespec date,
-                          const char *memo, const char *num)
+gncOwnerCreatePaymentLotSecs (const GncOwner *owner, Transaction **preset_txn,
+                              Account *posted_acc, Account *xfer_acc,
+                              gnc_numeric amount, gnc_numeric exch, time64 date,
+                              const char *memo, const char *num)
 {
     QofBook *book;
     Split *split;
     const char *name;
-    gnc_commodity *commodity;
+    gnc_commodity *post_comm, *xfer_comm;
     Split *xfer_split = NULL;
     Transaction *txn = NULL;
     GNCLot *payment_lot;
+    gnc_numeric xfer_amount = gnc_numeric_zero();
+    gnc_numeric txn_value = gnc_numeric_zero();
 
     /* Verify our arguments */
     if (!owner || !posted_acc || !xfer_acc) return NULL;
@@ -743,7 +769,9 @@ gncOwnerCreatePaymentLot (const GncOwner *owner, Transaction **preset_txn,
     /* Compute the ancillary data */
     book = gnc_account_get_book (posted_acc);
     name = gncOwnerGetName (gncOwnerGetEndOwner ((GncOwner*)owner));
-    commodity = gncOwnerGetCurrency (owner);
+    post_comm = xaccAccountGetCommodity (posted_acc);
+    xfer_comm = xaccAccountGetCommodity (xfer_acc);
+
 //    reverse = use_reversed_payment_amounts(owner);
 
     if (preset_txn && *preset_txn)
@@ -751,112 +779,90 @@ gncOwnerCreatePaymentLot (const GncOwner *owner, Transaction **preset_txn,
 
     if (txn)
     {
-        xaccTransSetDescription (txn, name ? name : "");
+        int i = 0;
 
         /* Pre-existing transaction was specified. We completely clear it,
-         * except for the split in the transfer account, unless the
-         * transaction can't be reused (wrong currency, wrong transfer account).
-         * In that case, the transaction is simply removed and an new
-         * one created. */
-
+         * except for a pre-existing transfer split. We're very conservative
+         * in preserving that one as it may have been reconciled already. */
         xfer_split = xaccTransFindSplitByAccount(txn, xfer_acc);
-
-        if (xaccTransGetCurrency(txn) != gncOwnerGetCurrency (owner))
+        xaccTransBeginEdit (txn);
+        while (i < xaccTransCountSplits(txn))
         {
-            g_message("Uh oh, mismatching currency/commodity between selected transaction and owner. We fall back to manual creation of a new transaction.");
-            xfer_split = NULL;
-        }
-
-        if (!xfer_split)
-        {
-            g_message("Huh? Asset account not found anymore. Fully deleting old txn and now creating a new one.");
-
-            xaccTransBeginEdit (txn);
-            xaccTransDestroy (txn);
-            xaccTransCommitEdit (txn);
-
-            txn = NULL;
-        }
-        else
-        {
-            int i = 0;
-            xaccTransBeginEdit (txn);
-            while (i < xaccTransCountSplits(txn))
-            {
-                Split *split = xaccTransGetSplit (txn, i);
-                if (split == xfer_split)
-                {
-                    gnc_set_num_action (NULL, split, num, _("Payment"));
-                    ++i;
-                }
-                else
-                {
-                    xaccSplitDestroy(split);
-                }
-            }
-            /* Note: don't commit transaction now - that would insert an imbalance split.*/
+            Split *split = xaccTransGetSplit (txn, i);
+            if (split == xfer_split)
+                ++i;
+            else
+                xaccSplitDestroy(split);
         }
+        /* Note: don't commit transaction now - that would insert an imbalance split.*/
     }
-
-    /* Create the transaction if we don't have one yet */
-    if (!txn)
+    else
     {
         txn = xaccMallocTransaction (book);
         xaccTransBeginEdit (txn);
     }
 
+    /* Complete transaction setup */
+    xaccTransSetDescription (txn, name ? name : "");
+    if (!gnc_commodity_equal(xaccTransGetCurrency (txn), post_comm) &&
+        !gnc_commodity_equal (xaccTransGetCurrency (txn), xfer_comm))
+        xaccTransSetCurrency (txn, xfer_comm);
+
+    /* With all commodities involved known, define split amounts and txn value.
+     * - post amount (amount passed in as parameter) is always in post_acct commodity,
+     * - xfer amount requires conversion if the xfer account has a different
+     *   commodity than the post account.
+     * - txn value requires conversion if the post account has a different
+     *   commodity than the transaction */
+    if (gnc_commodity_equal(post_comm, xfer_comm))
+        xfer_amount = amount;
+    else
+        xfer_amount = gnc_numeric_mul (amount, exch, GNC_DENOM_AUTO,
+                                       GNC_HOW_RND_ROUND_HALF_UP);
+
+    if (gnc_commodity_equal(post_comm, xaccTransGetCurrency (txn)))
+        txn_value = amount;
+    else
+        txn_value = gnc_numeric_mul (amount, exch, GNC_DENOM_AUTO,
+                                      GNC_HOW_RND_ROUND_HALF_UP);
+
     /* Insert a split for the transfer account if we don't have one yet */
     if (!xfer_split)
     {
-
-        /* Set up the transaction */
-        xaccTransSetDescription (txn, name ? name : "");
-        /* set per book option */
-        xaccTransSetCurrency (txn, commodity);
-        xaccTransSetDateEnteredSecs (txn, gnc_time (NULL));
-        xaccTransSetDatePostedTS (txn, &date);
-
-
         /* The split for the transfer account */
-        split = xaccMallocSplit (book);
-        xaccSplitSetMemo (split, memo);
+        xfer_split = xaccMallocSplit (book);
+        xaccSplitSetMemo (xfer_split, memo);
         /* set per book option */
-        gnc_set_num_action (NULL, split, num, _("Payment"));
+        gnc_set_num_action (NULL, xfer_split, num, _("Payment"));
         xaccAccountBeginEdit (xfer_acc);
-        xaccAccountInsertSplit (xfer_acc, split);
+        xaccAccountInsertSplit (xfer_acc, xfer_split);
         xaccAccountCommitEdit (xfer_acc);
-        xaccTransAppendSplit (txn, split);
+        xaccTransAppendSplit (txn, xfer_split);
 
-        if (gnc_commodity_equal(xaccAccountGetCommodity(xfer_acc), commodity))
-        {
-            xaccSplitSetBaseValue (split, amount, commodity);
-        }
-        else
-        {
-            /* This will be a multi-currency transaction. The amount passed to this
-             * function is in the owner commodity (also used by the post account).
-             * For the xfer split we also need to value the payment in the xfer account's
-             * commodity.
-             * exch is from post account to xfer account so that can be used directly
-             * to calculate the equivalent amount in the xfer account's commodity. */
-            gnc_numeric xfer_amount = gnc_numeric_mul (amount, exch, GNC_DENOM_AUTO,
-                                                 GNC_HOW_RND_ROUND_HALF_UP);
-
-            xaccSplitSetAmount(split, xfer_amount); /* Payment in xfer account currency */
-            xaccSplitSetValue(split, amount); /* Payment in transaction currency */
-        }
+        xaccSplitSetAmount(xfer_split, xfer_amount); /* Payment in xfer account currency */
+        xaccSplitSetValue(xfer_split, txn_value); /* Payment in transaction currency */
+    }
+    /* For a pre-existing xfer split, let's check if the amount and value
+     * have changed. If so, update them and unreconcile. */
+    else if (!gnc_numeric_equal (xaccSplitGetAmount (xfer_split), xfer_amount) ||
+             !gnc_numeric_equal (xaccSplitGetValue (xfer_split), txn_value))
+    {
+        xaccSplitSetAmount (xfer_split, xfer_amount);
+        xaccSplitSetValue (xfer_split, txn_value);
+        xaccSplitSetReconcile (xfer_split, NREC);
     }
 
     /* Add a split in the post account */
     split = xaccMallocSplit (book);
     xaccSplitSetMemo (split, memo);
     /* set per book option */
-    gnc_set_num_action (NULL, split, num, _("Payment"));
+    xaccSplitSetAction (split, _("Payment"));
     xaccAccountBeginEdit (posted_acc);
     xaccAccountInsertSplit (posted_acc, split);
     xaccAccountCommitEdit (posted_acc);
     xaccTransAppendSplit (txn, split);
-    xaccSplitSetBaseValue (split, gnc_numeric_neg (amount), commodity);
+    xaccSplitSetAmount (split, gnc_numeric_neg (amount));
+    xaccSplitSetValue (split, gnc_numeric_neg (txn_value));
 
     /* Create a new lot for the payment */
     payment_lot = gnc_lot_new (book);
@@ -864,9 +870,13 @@ gncOwnerCreatePaymentLot (const GncOwner *owner, Transaction **preset_txn,
     gnc_lot_add_split (payment_lot, split);
 
     /* Mark the transaction as a payment */
-    gnc_set_num_action (txn, NULL, num, _("Payment"));
+    xaccTransSetNum (txn, num);
     xaccTransSetTxnType (txn, TXN_TYPE_PAYMENT);
 
+    /* Set date for transaction */
+    xaccTransSetDateEnteredSecs (txn, gnc_time (NULL));
+    xaccTransSetDatePostedSecs (txn, date);
+
     /* Commit this new transaction */
     xaccTransCommitEdit (txn);
     if (preset_txn)
@@ -1018,13 +1028,15 @@ gncOwnerSetLotLinkMemo (Transaction *ll_txn)
 
         title = g_strdup_printf ("%s %s", gncInvoiceGetTypeString (invoice), gncInvoiceGetID (invoice));
 
-        titles = g_list_insert_sorted (titles, title, (GCompareFunc)g_strcmp0);
+        titles = g_list_prepend (titles, title);
         splits = g_list_prepend (splits, split); // splits don't need to be sorted
     }
 
     if (!titles)
         return; // We didn't find document lots
 
+    titles = g_list_sort (titles, (GCompareFunc)g_strcmp0);
+
     // Create the memo as we'd want it to be
     new_memo = g_strconcat (memo_prefix, titles->data, NULL);
     for (titer = titles->next; titer; titer = titer->next)
@@ -1100,7 +1112,7 @@ gncOwnerCreateLotLink (GNCLot *from_lot, GNCLot *to_lot, const GncOwner *owner)
     const gchar *name = gncOwnerGetName (gncOwnerGetEndOwner (owner));
     Transaction *ll_txn = NULL;
     gnc_numeric from_lot_bal, to_lot_bal;
-    Timespec from_ts, to_ts;
+    time64 from_time, to_time;
     time64 time_posted;
     Split *split;
 
@@ -1110,12 +1122,12 @@ gncOwnerCreateLotLink (GNCLot *from_lot, GNCLot *to_lot, const GncOwner *owner)
         return;
 
     /* Determine transaction date based on lot splits */
-    from_ts = xaccTransRetDatePostedTS (xaccSplitGetParent (gnc_lot_get_latest_split (from_lot)));
-    to_ts   = xaccTransRetDatePostedTS (xaccSplitGetParent (gnc_lot_get_latest_split (to_lot)));
-    if (timespecToTime64 (from_ts) >= timespecToTime64 (to_ts))
-        time_posted = timespecToTime64 (from_ts);
+    from_time = xaccTransRetDatePosted (xaccSplitGetParent (gnc_lot_get_latest_split (from_lot)));
+    to_time   = xaccTransRetDatePosted (xaccSplitGetParent (gnc_lot_get_latest_split (to_lot)));
+    if (from_time >= to_time)
+        time_posted = from_time;
     else
-        time_posted = timespecToTime64 (to_ts);
+        time_posted = to_time;
 
     /* Figure out how much we can offset between the lots */
     from_lot_bal = gnc_lot_get_balance (from_lot);
@@ -1137,31 +1149,22 @@ gncOwnerCreateLotLink (GNCLot *from_lot, GNCLot *to_lot, const GncOwner *owner)
     if (!ll_txn)
     {
         /* No pre-existing lot link. Create one. */
-        Timespec ts;
-
-        timespecFromTime64 (&ts, time_posted);
-
         ll_txn = xaccMallocTransaction (gnc_lot_get_book (from_lot));
         xaccTransBeginEdit (ll_txn);
-
         xaccTransSetDescription (ll_txn, name ? name : "(Unknown)");
         xaccTransSetCurrency (ll_txn, xaccAccountGetCommodity(acct));
         xaccTransSetDateEnteredSecs (ll_txn, gnc_time (NULL));
-        xaccTransSetDatePostedTS (ll_txn, &ts);
+        xaccTransSetDatePostedSecs (ll_txn, time_posted);
         xaccTransSetTxnType (ll_txn, TXN_TYPE_LINK);
     }
     else
     {
-        Timespec ts = xaccTransRetDatePostedTS (ll_txn);
+        time64 time = xaccTransRetDatePosted (ll_txn);
         xaccTransBeginEdit (ll_txn);
 
         /* Maybe we need to update the post date of the transaction ? */
-        if (time_posted > timespecToTime64 (ts))
-        {
-            timespecFromTime64 (&ts, time_posted);
-            xaccTransSetDatePostedTS (ll_txn, &ts);
-
-        }
+        if (time_posted > time)
+            xaccTransSetDatePostedSecs (ll_txn, time_posted);
     }
 
     /* Create a split for the from_lot */
@@ -1385,10 +1388,10 @@ void gncOwnerAutoApplyPaymentsWithLots (const GncOwner *owner, GList *lots)
  * then all open lots for the owner are considered.
  */
 void
-gncOwnerApplyPayment (const GncOwner *owner, Transaction **preset_txn, GList *lots,
-                      Account *posted_acc, Account *xfer_acc,
-                      gnc_numeric amount, gnc_numeric exch, Timespec date,
-                      const char *memo, const char *num, gboolean auto_pay)
+gncOwnerApplyPaymentSecs (const GncOwner *owner, Transaction **preset_txn,
+                          GList *lots, Account *posted_acc, Account *xfer_acc,
+                          gnc_numeric amount, gnc_numeric exch, time64 date,
+                          const char *memo, const char *num, gboolean auto_pay)
 {
     GNCLot *payment_lot = NULL;
     GList *selected_lots = NULL;
@@ -1400,8 +1403,10 @@ gncOwnerApplyPayment (const GncOwner *owner, Transaction **preset_txn, GList *lo
 
     /* If there's a real amount to transfer create a lot for this payment */
     if (!gnc_numeric_zero_p (amount))
-        payment_lot = gncOwnerCreatePaymentLot (owner, preset_txn, posted_acc, xfer_acc,
-                                                amount, exch, date, memo, num);
+        payment_lot = gncOwnerCreatePaymentLotSecs (owner, preset_txn,
+                                                    posted_acc, xfer_acc,
+                                                    amount, exch, date, memo,
+                                                    num);
 
     if (lots)
         selected_lots = lots;
@@ -1458,46 +1463,60 @@ gncOwnerGetBalanceInCurrency (const GncOwner *owner,
                               const gnc_commodity *report_currency)
 {
     gnc_numeric balance = gnc_numeric_zero ();
-    GList *acct_list, *acct_node, *acct_types, *lot_list = NULL, *lot_node;
     QofBook *book;
     gnc_commodity *owner_currency;
     GNCPriceDB *pdb;
+    const gnc_numeric *cached_balance = NULL;
 
     g_return_val_if_fail (owner, gnc_numeric_zero ());
 
-    /* Get account list */
     book       = qof_instance_get_book (qofOwnerGetOwner (owner));
-    acct_list  = gnc_account_get_descendants (gnc_book_get_root_account (book));
-    acct_types = gncOwnerGetAccountTypesList (owner);
     owner_currency = gncOwnerGetCurrency (owner);
 
-    /* For each account */
-    for (acct_node = acct_list; acct_node; acct_node = acct_node->next)
+    cached_balance = gncOwnerGetCachedBalance (owner);
+    if (cached_balance)
+        balance = *cached_balance;
+    else
     {
-        Account *account = acct_node->data;
+        /* No valid cache value found for balance. Let's recalculate */
+        GList *acct_list  = gnc_account_get_descendants (gnc_book_get_root_account (book));
+        GList *acct_types = gncOwnerGetAccountTypesList (owner);
+        GList *acct_node;
 
-        /* Check if this account can have lots for the owner, otherwise skip to next */
-        if (g_list_index (acct_types, (gpointer)xaccAccountGetType (account))
-                == -1)
-            continue;
-
-
-        if (!gnc_commodity_equal (owner_currency, xaccAccountGetCommodity (account)))
-            continue;
-
-        /* Get a list of open lots for this owner and account */
-        lot_list = xaccAccountFindOpenLots (account, gncOwnerLotMatchOwnerFunc,
-                                            (gpointer)owner, NULL);
-        /* For each lot */
-        for (lot_node = lot_list; lot_node; lot_node = lot_node->next)
+        /* For each account */
+        for (acct_node = acct_list; acct_node; acct_node = acct_node->next)
         {
-            GNCLot *lot = lot_node->data;
-            gnc_numeric lot_balance = gnc_lot_get_balance (lot);
-            GncInvoice *invoice = gncInvoiceGetInvoiceFromLot(lot);
-            if (invoice)
-               balance = gnc_numeric_add (balance, lot_balance,
-                                          gnc_commodity_get_fraction (owner_currency), GNC_HOW_RND_ROUND_HALF_UP);
+            Account *account = acct_node->data;
+            GList *lot_list = NULL, *lot_node;
+
+            /* Check if this account can have lots for the owner, otherwise skip to next */
+            if (g_list_index (acct_types, (gpointer)xaccAccountGetType (account))
+                    == -1)
+                continue;
+
+
+            if (!gnc_commodity_equal (owner_currency, xaccAccountGetCommodity (account)))
+                continue;
+
+            /* Get a list of open lots for this owner and account */
+            lot_list = xaccAccountFindOpenLots (account, gncOwnerLotMatchOwnerFunc,
+                                                (gpointer)owner, NULL);
+            /* For each lot */
+            for (lot_node = lot_list; lot_node; lot_node = lot_node->next)
+            {
+                GNCLot *lot = lot_node->data;
+                gnc_numeric lot_balance = gnc_lot_get_balance (lot);
+                GncInvoice *invoice = gncInvoiceGetInvoiceFromLot(lot);
+                if (invoice)
+                balance = gnc_numeric_add (balance, lot_balance,
+                                            gnc_commodity_get_fraction (owner_currency), GNC_HOW_RND_ROUND_HALF_UP);
+            }
+            g_list_free (lot_list);
         }
+        g_list_free (acct_list);
+        g_list_free (acct_types);
+
+        gncOwnerSetCachedBalance (owner, &balance);
     }
 
     pdb = gnc_pricedb_get_db (book);
@@ -1589,3 +1608,30 @@ gboolean gncOwnerRegister (void)
 
     return TRUE;
 }
+
+const gnc_numeric*
+gncOwnerGetCachedBalance (const GncOwner *owner)
+{
+    if (!owner) return NULL;
+
+    if (gncOwnerGetType (owner) == GNC_OWNER_CUSTOMER)
+        return gncCustomerGetCachedBalance (gncOwnerGetCustomer (owner));
+    else if (gncOwnerGetType (owner) == GNC_OWNER_VENDOR)
+        return gncVendorGetCachedBalance (gncOwnerGetVendor (owner));
+    else if (gncOwnerGetType (owner) == GNC_OWNER_EMPLOYEE)
+        return gncEmployeeGetCachedBalance (gncOwnerGetEmployee (owner));
+
+    return NULL;
+}
+
+void gncOwnerSetCachedBalance (const GncOwner *owner, const gnc_numeric *new_bal)
+{
+    if (!owner) return;
+
+    if (gncOwnerGetType (owner) == GNC_OWNER_CUSTOMER)
+        gncCustomerSetCachedBalance (gncOwnerGetCustomer (owner), new_bal);
+    else if (gncOwnerGetType (owner) == GNC_OWNER_VENDOR)
+        gncVendorSetCachedBalance (gncOwnerGetVendor (owner), new_bal);
+    else if (gncOwnerGetType (owner) == GNC_OWNER_EMPLOYEE)
+        gncEmployeeSetCachedBalance (gncOwnerGetEmployee (owner), new_bal);
+}
